#include <stdio.h>
#include <conio.h>

#define CH_BEEP        7
#define CH_BACKSPACE   8
#define CH_RETURN     10
#define CH_AT         22
#define CH_SZ        126
#define CH_DEL_LEFT  127


typedef struct {
    char name[40];
    char vorname[40];
    char strasse[40];
    char plz[6];
    char wohnort[40];
    char telefon[20];
    char email[30];
} t_address;


/* Input a string with up to maxlen characters via keyboard.
   Parameters: cp      pointer to character buffer
               maxlen  maximum input length allowed
 */
void inputString(char *cp, int maxlen) {
    char key;
    char *ptrBuf;
    int numChars;
    
    ptrBuf = cp;
    *ptrBuf = '\0'; // we start with an empty string
    numChars = 0;
    
    for (;;) {
        key = getch();
        
        if (key == CH_RETURN) { // finish input if <return> pressed
            putchar('\n');
            break;
        }
        
        if (numChars < maxlen) {
            if (key >  31 && key < 126 || // regular characters
                key >  90 && key <  94 || // uppercase Umlaut
                key > 122 && key < 126 || // lowercase Umlaut
//              key == CH_AT ||
                key == CH_SZ)
            {
                putchar(key);    // print character on screen
                *ptrBuf++ = key; // write key to buffer
                *ptrBuf = '\0';  // finalise string and point to the '\0' character
                ++numChars;
            }
            else {
                if (key == CH_DEL_LEFT && *cp != '\0') { // delete left and characters available
                    putchar(CH_BACKSPACE); // move left
                    putchar(' ');          // overwrite with blank
                    putchar(CH_BACKSPACE); // move left again
                    *--ptrBuf = '\0';      // shortens the string by 1 char
                    --numChars;
                }
                else {
                    putchar(CH_BEEP); // invalid key typed => beep
                }
            }
        }
        else {
            putchar(CH_BEEP); // maximum number of characters reached => beep
        }
    }
}

/* Main
 */
int main(void) {
    t_address address;
    FILE *fp;
    char key;

    fp = fopen("adressen.txt", "a"); // open the file in "append" mode
    if (fp == NULL) {
        puts("Fehler: Konnte die Ausgabedatei adressen.txt nicht oeffnen!");
        return -1;
    }

    // Input loop
    do {
        // Input address
        printf("Name:    "); inputString(address.name,    39);
        printf("Vorname: "); inputString(address.vorname, 39);
        printf("Strasse: "); inputString(address.strasse, 39);
        printf("PLZ:     "); inputString(address.plz,      5);
        printf("Wohnort: "); inputString(address.wohnort, 39);
        printf("Telefon: "); inputString(address.telefon, 19);
        printf("E-Mail:  "); inputString(address.email,   29);

        // Write address to file
        fprintf(fp, "%s %s\r", address.name, address.vorname);
        fprintf(fp, "%s\r", address.strasse);
        fprintf(fp, "%s %s\r", address.plz, address.wohnort);
        fprintf(fp, "%s\r", address.telefon);
        fprintf(fp, "%s\r", address.email);
        
        // Add another address?
        printf("\nWeitere Adresse eingeben? (J/N) ");
        key = getch();
    } while (key == 'J' || key == 'j');

    putchar('\n');

    fclose(fp);

    return 0;
}
